import unittest

from bgez.asyn import LoopTestCase, Promise
from bgez import loop

class EventLoopTest(LoopTestCase):

    def test_async_def(self):
        resolve_to = object()
        called = True

        @loop.task
        async def test():
            nonlocal called
            called = True
            return resolve_to

        promise = test()

        self.assertEqual(resolve_to, promise.peek())

        @self.postLoop
        def _after_loop():
            self.assertTrue(called)

    def test_async_await(self):
        resolve_to = object()
        called = False

        a = Promise.resolved(resolve_to)
        self.assertEqual(resolve_to, a.peek())

        @loop.task
        async def test():
            nonlocal called

            value = await a
            self.assertEqual(value, resolve_to)
            called = True

        promise = test()

        @self.postLoop
        def _after_loop():
            self.assertEqual(None, promise.peek())
            self.assertTrue(called)

    def test_value_converter(self):
        resolve_to = object()
        called = False

        @loop.task
        async def test():
            nonlocal called
            value = await loop.value(resolve_to)
            self.assertEqual(value, resolve_to)
            called = True

        promise = test()

        @self.postLoop
        def _after_loop():
            self.assertEqual(None, promise.peek())
            self.assertTrue(called)
