from typing import Any, Callable

from inspect import Parameter, signature
from functools import wraps, partial

__all__ = [
    'decorator',
]

class decorator:
    '''
    Allow last positionnal arguement to be left undefined,
     the function will become a decorator then.
    '''
    def __init__(self, function: Callable, position: int = None):
        wraps(function)(self)

        if position is None:
            position = self.__guessPosition(function)

        self.__position = position
        self.__wrapped__ = function

    def __guessPosition(self, function):
        for i, (_, parameter) in enumerate(signature(function).parameters.items()):
            if parameter.kind not in (Parameter.POSITIONAL_ONLY, Parameter.POSITIONAL_OR_KEYWORD):
                if i == 0: break # no positional arguments
                return i - 1
        else:
            return i
        raise AttributeError(f'{function} has no positional arguments')

    def __call__(self, *args, **kargs) -> Any:
        position = self.__position
        length = len(args)

        if position == length:
            return partial(self.__wrapped__, *args, **kargs)
        elif position == length - 1 and args[position] is None:
            return partial(self.__wrapped__, *args[:-1], **kargs)
        return self.__wrapped__(*args, **kargs)

    def __get__(self, *args) -> Callable:
        return type(self)(self.__wrapped__.__get__(*args))
