import time

from bgez.core.abstract import Abstract, abstractmethod

__all__ = [
    'BaseClock',
    'TimeClock',
    'TickClock',
]

class BaseClock(Abstract):

    @abstractmethod
    def time(self):
        '''Returns the time for this clock'''

    def tick(self):
        '''Updates the time if required, NO-OP by default.'''

class TimeClock(BaseClock):

    def __init__(self):
        ...

    def time(self):
        return time.monotonic()

class TickClock(BaseClock):

    def __init__(self):
        self._counter = 0

    def time(self):
        return self._counter

    def tick(self):
        self._counter += 1
